/* This is the driver file for:
 *
 * tnzs:     The New Zealand Story
 * tnzs_alt: The New Zealand Story (alternate)
 * tnzs_new: The New Zealand Story (new set)
 *
 */

#include <string.h>
#include "nz.h"
#include "soundhw.h"
#include "raze.h"

void tnzs_skipidle(word port, byte value);
byte tnzs_readmem_snd(word addr);
byte tnzs_readmem_input(word addr);
void tnzs_writemem_pal(word addr, byte value);
void tnzs_writemem_banksw0(word addr, byte value);
void tnzs_writemem_input(word addr, byte value);
void tnzs_writemem_snd(word addr, byte value);
void tnzs_writemem_banksw1(word addr, byte value);
int TNZS_LoadGraphics(void);
void TNZS_LoadHighScores(void);
void TNZS_SaveHighScores(void);
void TNZS_DrawColumns(BITMAP *bmp, int upperbits, int start, int end);
byte TNZS_PortHandler(int chip, int port, int iswrite, byte val);

static int tnzs_banknum0, tnzs_banknum1;
static byte tnzs_bankram[2 * 0x4000], *tnzs_bank0, *tnzs_bank1;

static int tnzs_input_count, tnzs_credit_count, tnzs_inserting_coin;
static BITMAP *tnzs_columns;
static int need_pal_recalc;
static int highscore_ok, highscore_loaded;

extern GAME tnzs_alt_driver;
extern GAME tnzs_new_driver;

static byte highscore[0x23];

int TNZS_Init(void)
{
   FILE *hi;
   GUI_PrintLog(FALSE, "Initialising TNZS driver.");

   if (current_game == &tnzs_alt_driver)
      hi = fopen("hiscore.alt", "rb");
   else
      hi = fopen("hiscore.nzs", "rb");

   if (hi) {
      fread(&highscore[0], 1, 0x23, hi);
      fclose(hi);
      highscore_loaded = TRUE;
      GUI_PrintLog(FALSE, "High-scores have been loaded.");
   } else highscore_loaded = FALSE;

   tnzs_columns = NULL;
   tnzs_credit_count = 0;
   tnzs_inserting_coin = FALSE;
   machine->cpu[0].image = machine->cpu[1].image = NULL;

   GUI_PrintLog(FALSE, "Setting up CPU 0.");
   machine->cpu[0].image = (byte *)malloc(0x10000);
   if (!machine->cpu[0].image)
   {
      GUI_PrintLog(TRUE, "Out of memory for 64K image (CPU 0).");
      GUI_MessageBox(MB_OK, NULL, "Out of memory for 64K image (CPU 0).");
      return FALSE;
   }

   GUI_PrintLog(FALSE, "Setting up CPU 1.");
   machine->cpu[1].image = (byte *)malloc(0x10000);
   if (!machine->cpu[1].image)
   {
      GUI_PrintLog(TRUE, "Out of memory for 64K image (CPU 1).");
      GUI_MessageBox(MB_OK, NULL, "Out of memory for 64K image (CPU 1).");
      return FALSE;
   }

   if (!TNZS_LoadGraphics()) return FALSE;

   GUI_PrintLog(FALSE, "Creating background bitmap.");
   tnzs_columns = create_bitmap(32*16, 16*16);
   if (!tnzs_columns)
   {
      GUI_PrintLog(TRUE, "Out of memory for background columns.");
      GUI_MessageBox(MB_OK, NULL, "Out of memory for background columns.");
      return FALSE;
   }

   ym2203_SetPortHandler(0, 0, &TNZS_PortHandler);
   ym2203_SetPortHandler(0, 1, &TNZS_PortHandler);

   return TRUE;
}

void TNZS_Reset(void)
{
   int n;
   GUI_PrintLog(FALSE, "Resetting TNZS hardware.");

   TNZS_SaveHighScores();

   if (current_game == &tnzs_new_driver)
   {
      memcpy(&(machine->cpu[0].image[0x0000]), &(current_game->rom_list[0].rom[0x0000]), 0x8000);
      memset(&(machine->cpu[0].image[0x8000]), 0, 0x8000);
      memcpy(&(machine->cpu[1].image[0x0000]), &(current_game->rom_list[2].rom[0x0000]), 0xa000);
      memset(&(machine->cpu[1].image[0xa000]), 0, 0x6000);
   } else {
      memcpy(&(machine->cpu[0].image[0x0000]), &(current_game->rom_list[0].rom[0x0000]), 0x8000);
      memset(&(machine->cpu[0].image[0x8000]), 0, 0x8000);
      memcpy(&(machine->cpu[1].image[0x0000]), &(current_game->rom_list[1].rom[0x0000]), 0xa000);
      memset(&(machine->cpu[1].image[0xa000]), 0, 0x6000);
   }

   if (current_game == &tnzs_alt_driver)
   {
      /* Patch ROMs for idle-skipping */
      machine->cpu[0].image[0x028c] = 0xd3; // OUT A, 0xaa */
      machine->cpu[0].image[0x028d] = 0xaa; // OUT A, 0xaa */
      machine->cpu[0].image[0x028e] = 0x00; // NOP */
      machine->cpu[0].image[0x028f] = 0x00; // NOP */
      machine->cpu[0].image[0x0290] = 0x00; // NOP */
      machine->cpu[0].image[0x0291] = 0x00; // NOP */
      machine->cpu[0].image[0x0292] = 0x00; // NOP */

      machine->cpu[1].image[0x013e] = 0xd3; // OUT A, 0xaa */
      machine->cpu[1].image[0x013f] = 0xaa; // OUT A, 0xaa */
      machine->cpu[1].image[0x0140] = 0xc3; // JP 0x0139 */
      machine->cpu[1].image[0x0141] = 0x39; // JP 0x0139 */
      machine->cpu[1].image[0x0142] = 0x01; // JP 0x0139 */
   } else {
      /* Patch ROMs for idle-skipping */
      machine->cpu[0].image[0x02db] = 0xd3; // OUT A, 0xaa */
      machine->cpu[0].image[0x02dc] = 0xaa; // OUT A, 0xaa */
      machine->cpu[0].image[0x02dd] = 0x00; // NOP */
      machine->cpu[0].image[0x02de] = 0x00; // NOP */
      machine->cpu[0].image[0x02df] = 0x00; // NOP */
      machine->cpu[0].image[0x02e0] = 0x00; // NOP */
      machine->cpu[0].image[0x02e1] = 0x00; // NOP */

      machine->cpu[1].image[0x0018] = 0xd3; // OUT A, 0xaa */
      machine->cpu[1].image[0x0019] = 0xaa; // OUT A, 0xaa */
      machine->cpu[1].image[0x001a] = 0xc3; // JP 0x0013 */
      machine->cpu[1].image[0x001b] = 0x13; // JP 0x0013 */

      /* Patch ROM checksums */
      machine->cpu[0].image[0x0cb6] = 0xc9; // RET */
      machine->cpu[1].image[0x0692] = 0xc9; // RET */

      /* Patch sound command write-check */
      machine->cpu[0].image[0x0c67] = 0xc9; // RET */
   }

   highscore_ok = FALSE;
   tnzs_input_count = -3;
   tnzs_credit_count = 0;
   tnzs_banknum0 = tnzs_banknum1 = -1;
   tnzs_bank0 = &(tnzs_bankram[0x0000]);
   tnzs_bank1 = &(machine->cpu[1].image[0x8000]);

   z80_set_context(machine->cpu[0].regs);
   z80_init_memmap();
   z80_map_fetch(0x0000, 0xffff, machine->cpu[0].image);
   z80_map_read( 0x0000, 0xffff, machine->cpu[0].image);
   z80_map_write(0x0000, 0xffff, machine->cpu[0].image);

   /* CPU0: set bank 0 (RAM) */
   z80_map_fetch(0x8000, 0xbfff, &tnzs_bankram[0]);
   z80_map_read( 0x8000, 0xbfff, &tnzs_bankram[0]);
   z80_map_write(0x8000, 0xbfff, &tnzs_bankram[0]);

   z80_add_write(0xf600, 0xf6ff, Z80_MAP_HANDLED, &tnzs_writemem_banksw0);
   z80_add_write(0xf800, 0xfbff, Z80_MAP_HANDLED, &tnzs_writemem_pal);

   z80_set_out(&tnzs_skipidle);
   z80_end_memmap();
   z80_get_context(machine->cpu[0].regs);

   z80_set_context(machine->cpu[1].regs);
   z80_init_memmap();
   /* CPU1: page in the first 32K ROM, 8K banked ROM, and 24K RAM */
   z80_map_fetch(0x0000, 0xffff, machine->cpu[1].image);
   z80_map_read( 0x0000, 0x9fff, machine->cpu[1].image);
   z80_map_read( 0xd000, 0xdfff, &(machine->cpu[1].image[0xd000]));
   z80_map_write(0xd000, 0xdfff, &(machine->cpu[1].image[0xd000]));

   /* CPU1: Map 0xe000 to CPU0 */
   z80_map_fetch(0xe000, 0xefff, &(machine->cpu[0].image[0xe000]));
   z80_map_read( 0xe000, 0xefff, &(machine->cpu[0].image[0xe000]));
   z80_map_write(0xe000, 0xefff, &(machine->cpu[0].image[0xe000]));

   z80_add_read( 0xb000, 0xbfff, Z80_MAP_HANDLED, &tnzs_readmem_snd);
   z80_add_read( 0xc000, 0xcfff, Z80_MAP_HANDLED, &tnzs_readmem_input);
   z80_add_write(0xa000, 0xafff, Z80_MAP_HANDLED, &tnzs_writemem_banksw1);
   z80_add_write(0xb000, 0xbfff, Z80_MAP_HANDLED, &tnzs_writemem_snd);
   z80_add_write(0xc000, 0xcfff, Z80_MAP_HANDLED, &tnzs_writemem_input);

   z80_set_out(&tnzs_skipidle);
   z80_end_memmap();
   z80_get_context(machine->cpu[1].regs);

   /* set the palette to random */
   for (n=0;n<512;n++)
   {
      machine->cpu[0].image[0xf800+n*2] = rand()&255;
      machine->cpu[0].image[0xf800+n*2+1] = rand()&255;
   }
   need_pal_recalc = TRUE;
}

void TNZS_Shutdown(void)
{
   if (tnzs_columns) destroy_bitmap(tnzs_columns);
   tnzs_columns = NULL;
   TNZS_SaveHighScores();
}

void TNZS_LoadHighScores(void)
{
   word addr;
   if (current_game == &tnzs_alt_driver)
      addr = 0xec0a;
   else
      addr = 0xe68d;

   if (memcmp(&machine->cpu[0].image[addr+0x20], "GUU", 3) == 0)
   {
      if (highscore_loaded) {
         memcpy(&machine->cpu[0].image[addr], &highscore[0], 0x23);
         GUI_PrintLog(FALSE, "High-scores have been updated.");
      } else GUI_PrintLog(FALSE, "High-scores finalised.");
      highscore_ok = TRUE;
   }
}

void TNZS_SaveHighScores(void)
{
   FILE *hi;
   word addr;
   if (highscore_ok) {
      if (current_game == &tnzs_alt_driver) {
         hi = fopen("hiscore.alt", "wb");
         addr = 0xec0a;
      } else {
         addr = 0xe68d;
         hi = fopen("hiscore.nzs", "wb");
      }
      memcpy(&highscore[0], &machine->cpu[0].image[addr], 0x23);
      highscore_loaded = TRUE;
      fwrite(&machine->cpu[0].image[addr], 1, 0x23, hi);
      fclose(hi);
      GUI_PrintLog(FALSE, "Saved high-scores to disk.");
   }
}

int TNZS_LoadGraphics(void)
{
   GUI_PrintLog(FALSE, "Converting graphics.");
   GUI_SetMouseCursor(gui_data[GUI_BUSY].dat, 0, 0);

   sprites = create_bitmap(16, 4096*2*16);
   if (!sprites)
   {
      GUI_PrintLog(TRUE, "Out of memory for sprite bitmaps.");
      GUI_MessageBox(MB_OK, NULL, "Out of memory for sprite bitmaps.");
      return FALSE;
   }
   clear(sprites);
   
   if (current_game == &tnzs_new_driver)
   {
      /* Load the 1st quarter of the gfx ROMs (sprites) */
      Emu_AddPlane(sprites, current_game->rom_list[3].rom, 3, 0, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[5].rom, 2, 0, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[7].rom, 1, 0, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[9].rom, 0, 0, 64*1024);

      /* Load the 2nd quarter of the gfx ROMs (sprites) */
      Emu_AddPlane(sprites, current_game->rom_list[4].rom, 3, 2048, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[6].rom, 2, 2048, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[8].rom, 1, 2048, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[10].rom, 0, 2048, 64*1024);

      /* Load the 3rd quarter of the gfx ROMs (background tiles) */
      Emu_AddPlane(sprites, current_game->rom_list[11].rom, 3, 4096, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[13].rom, 2, 4096, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[15].rom, 1, 4096, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[17].rom, 0, 4096, 64*1024);

      /* Load the 4th quarter of the gfx ROMs (background tiles) */
      Emu_AddPlane(sprites, current_game->rom_list[12].rom, 3, 6144, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[14].rom, 2, 6144, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[16].rom, 1, 6144, 64*1024);
      Emu_AddPlane(sprites, current_game->rom_list[18].rom, 0, 6144, 64*1024);
   } else {
      /* Load the lower half of the gfx ROMs (sprites) */
      Emu_AddPlane(sprites, current_game->rom_list[2].rom, 3, 0, 128*1024);
      Emu_AddPlane(sprites, current_game->rom_list[3].rom, 2, 0, 128*1024);
      Emu_AddPlane(sprites, current_game->rom_list[4].rom, 1, 0, 128*1024);
      Emu_AddPlane(sprites, current_game->rom_list[5].rom, 0, 0, 128*1024);

      /* Load the upper half of the gfx ROMs (background tiles) */
      Emu_AddPlane(sprites, current_game->rom_list[6].rom, 3, 4096, 128*1024);
      Emu_AddPlane(sprites, current_game->rom_list[7].rom, 2, 4096, 128*1024);
      Emu_AddPlane(sprites, current_game->rom_list[8].rom, 1, 4096, 128*1024);
      Emu_AddPlane(sprites, current_game->rom_list[9].rom, 0, 4096, 128*1024);
   }

   GUI_SetMouseCursor(gui_data[GUI_CURSOR].dat, 1, 1);
   return TRUE;
}

void tnzs_skipidle(word port, byte value)
{
   z80_skip_idle();
}

byte tnzs_readmem_snd(word addr)
{
   if (addr == 0xb000) return ym2203_ReadStatus(0);
   else return ym2203_ReadReg(0, sound_latch);
}

byte tnzs_readmem_input(word addr)
{
   int ret;
   if (addr != 0xc000) return 1;
   switch (tnzs_input_count)
   {
      case -3: ret = 0x5a; break;
      case -2: ret = 0xa5; break;
      case -1: ret = 0x55; break;
      case 6: tnzs_input_count = 0; /* fall through to below */
      case 0:
         ret = tnzs_credit_count; /* no. of credits */
         break;
      case 1: ret = 0x04 | 0x40;
              if (key[key_start1]) ret |= 0x80;
              if (key[key_start2]) ret |= 0x08;
              if (joy[0].button[0].b) ret |= 0x10;
              if (joy[0].button[1].b) ret |= 0x20;
              if (key[key_b1]) ret |= 0x10;
              if (key[key_b2]) ret |= 0x20;
              break;
      case 2: ret = 0xff; break;
      case 3: ret = 0xff; break;
      case 4: ret = 0x10 | 0x20 | 0x40 | 0x80;
              if (joy[0].stick[0].axis[0].d1) ret |= 0x01;
              if (joy[0].stick[0].axis[0].d2) ret |= 0x02;
              if (joy[0].stick[0].axis[1].d1) ret |= 0x04;
              if (joy[0].stick[0].axis[1].d2) ret |= 0x08;
              if (key[key_left]) ret |= 0x01;
              if (key[key_right]) ret |= 0x02;
              if (key[key_up]) ret |= 0x04;
              if (key[key_down]) ret |= 0x08;
              break;
      case 5: ret = 0x10 | 0x20 | 0x40 | 0x80; break;
      default: ret = 0xff; break;
   }
   tnzs_input_count++;
   return ret;
}

void tnzs_writemem_pal(word addr, byte value)
{
   machine->cpu[0].image[addr] = value;
   need_pal_recalc = TRUE;
}

void tnzs_writemem_banksw0(word addr, byte value)
{
   machine->cpu[0].image[addr] = value;
   if ((value&7) == tnzs_banknum0) return;

   tnzs_banknum0 = value&7;
   if (tnzs_banknum0 <= 1)
   {
      tnzs_bank0 = &tnzs_bankram[tnzs_banknum0 * 0x4000];
   } else {
      if (current_game == &tnzs_new_driver)
      {
         if (tnzs_banknum0 < 4)
            tnzs_bank0 = &(current_game->rom_list[0].rom[tnzs_banknum0 * 0x4000]);
         else
            tnzs_bank0 = &(current_game->rom_list[1].rom[tnzs_banknum0 * 0x4000 - 0x10000]);
      } else {
         tnzs_bank0 = &(current_game->rom_list[0].rom[tnzs_banknum0 * 0x4000]);
      }
   }
   z80_map_fetch(0x8000, 0xbfff, tnzs_bank0);
   z80_map_read( 0x8000, 0xbfff, tnzs_bank0);
   z80_map_write(0x8000, 0xbfff, tnzs_bank0);
}

void tnzs_writemem_input(word addr, byte value)
{
   if (addr == 0xc000)
   {
      tnzs_input_count = (tnzs_input_count + 5) % 6;
      tnzs_credit_count += (signed char)(value);
   }
}

void tnzs_writemem_snd(word addr, byte value)
{
   if (addr == 0xb000) sound_latch = value;
   if (addr == 0xb001)
   {
      ym2203_WriteReg(0, sound_latch, value);
   }
}

void tnzs_writemem_banksw1(word addr, byte value)
{
   if ((value&3) == tnzs_banknum1) return;
         
   /* switch banks */
   tnzs_banknum1 = value&3;
   if (current_game == &tnzs_new_driver)
      tnzs_bank1 = &(current_game->rom_list[2].rom[tnzs_banknum1 * 0x2000 + 0x8000]);
   else
      tnzs_bank1 = &(current_game->rom_list[1].rom[tnzs_banknum1 * 0x2000 + 0x8000]);
   z80_map_fetch(0x8000, 0x9fff, tnzs_bank1);
   z80_map_read( 0x8000, 0x9fff, tnzs_bank1);
}

void TNZS_DrawScreen(BITMAP *bmp)
{
   byte *info;
   int n, sprite, column, x,y, colour, x_flip;

   if (!highscore_ok) TNZS_LoadHighScores();

   if (need_pal_recalc)
   {
      need_pal_recalc = FALSE;
      Palette_Recalc(&(machine->cpu[0].image[0xf800]));
   }

   /* check the hardware credit counter */
   if (key[key_coin])
   {
      if (!tnzs_inserting_coin)
      {
         tnzs_inserting_coin = TRUE;
         tnzs_credit_count++;
      }
   } else {
      tnzs_inserting_coin = FALSE;
   }

   clear(bmp);

   /* global screen flip */
   x_flip = machine->cpu[0].image[0xf300] & 0x40;

   /* update background playfield */
   info = &(machine->cpu[0].image[32*15]);
   for (column=7;column>=0;column--)
   {
      for (y=0;y<16;y++)
      {
         for (x=0;x<2;x++,info++)
         {
            sprite = info[0xc400] | ((info[0xd400] & 31) << 8);
            colour = info[0xd600] >> 3;
            n = sprite | (colour << 16);

            if (dirty_map[column*2+x][y] != n)
            {
               dirty_map[column*2+x][y] = n;

               Emu_Blit(sprites, tnzs_columns,  /* src, dest */
                        0, sprite*16,           /* sx, sy */
                        column*32+x*16, y*16,   /* dx, dy */
                        x_flip,                 /* x_flip */
                        colour*16);             /* colour */
               //textprintf(tnzs_columns, font, column*32+x*16, y*16, GUI_WHITE, "%.2x", info[0xd400]);                        
            }
         }
      }
      info -= 32*2;
   }

   /* update foreground playfield */
   for (column=15;column>=8;column--)
   {
      for (y=0;y<16;y++)
      {
         for (x=0;x<2;x++,info++)
         {
            sprite = info[0xc400] | ((info[0xd400] & 31) << 8);
            colour = info[0xd600] >> 3;
            n = sprite | (colour << 16);

            if (dirty_map[column*2+x][y] != n)
            {
               dirty_map[column*2+x][y] = n;

               Emu_Blit(sprites, tnzs_columns,  /* src, dest */
                        0, sprite*16,           /* sx, sy */
                        column*32+x*16, y*16,   /* dx, dy */
                        x_flip,                 /* x_flip */
                        colour*16);             /* colour */
            }
         }
      }
      info -= 32*2;
   }

   if (machine->cpu[0].image[0xf301] & 1) /* background enable */
   {
      if (machine->cpu[0].image[0xf301] & 8) /* if set then disable columns 9-15 */
      {            
         /* draw background playfield */
         TNZS_DrawColumns(bmp, machine->cpu[0].image[0xf302], 0, 7);
         /* draw column 8 (foreground) */
         TNZS_DrawColumns(bmp, machine->cpu[0].image[0xf303] << 7, 8, 8);
      } else {
         /* draw background playfield */
         TNZS_DrawColumns(bmp, machine->cpu[0].image[0xf302], 0, 7);
         /* draw foreground playfield */
         TNZS_DrawColumns(bmp, machine->cpu[0].image[0xf303], 8, 15);
      }
   }

   info = &(machine->cpu[0].image[511]);
   for (n=511;n>=0;n--,info--)
   {
      if (info[0xf000] >= 0xf8) continue;
      y = 226 - info[0xf000];

      x = info[0xc200];
      if (info[0xd200] & 1) x -= 256;
      colour = info[0xd200] >> 3;

      sprite = info[0xc000]
             +(info[0xd000]&31) * 256;

      x_flip = info[0xd000] & 0x80;
      if (machine->cpu[0].image[0xf300] & 0x40) /* global screen flip */
      {
         x_flip = !x_flip;
         y -= 4;
      }

      Emu_MaskedBlit(sprites, bmp,        /* src, dest */
                     0, sprite*16,        /* sx, sy */
                     x, y,                /* dx, dy */
                     x_flip,              /* x_flip */
                     info[0xd000] & 0x40, /* y_flip */
                     colour*16);          /* colour */
      //textprintf(bmp, font, x, y, GUI_WHITE, "%.2x", info[0xd000]);                        
   }
}

void TNZS_DrawColumns(BITMAP *bmp, int upperbits, int start, int end)
{
   byte *info;
   int x, y, count;
   info = &(machine->cpu[0].image[0xf200+16*end]);

   count = end-start+1;
   do {
      upperbits <<= 1;
      x = info[4] - (upperbits & 0x100);
      y = -(info[0] + 15);
      if (machine->cpu[0].image[0xf300] & 0x40) /* global screen flip */
      {
         y -= 2;
      }

      masked_blit(tnzs_columns, bmp, end*32, 0, x, y, 32, 256);
      masked_blit(tnzs_columns, bmp, end*32, 0, x, y+256, 32, 256);
      info -= 16;
      end--;
   } while (--count);
}

byte TNZS_PortHandler(int chip, int port, int iswrite, byte val)
{
   if (port == 0)
   {
      return (~dip_sw_1) & 0xff;
   } else {
      return (~dip_sw_2) & 0xff;
   }
}

ROM_LIST tnzs_romlist[] = {
   /* filename,      size,     checksum */
   { "nzsb5310.bin", 128*1024, 0x935a36b4 },
   { "nzsb5311.bin",  64*1024, 0xcc5f8183 },

   { "nzsb5320.bin", 128*1024, 0x1325159d },
   { "nzsb5322.bin", 128*1024, 0xa390cc10 },
   { "nzsb5318.bin", 128*1024, 0x9f5d4531 },
   { "nzsb5316.bin", 128*1024, 0x29e31231 },

   { "nzsb5321.bin", 128*1024, 0xd29db545 },
   { "nzsb5323.bin", 128*1024, 0xc8841c06 },
   { "nzsb5319.bin", 128*1024, 0x045a4abc },
   { "nzsb5317.bin", 128*1024, 0x0b69d21b },
   { NULL }
};   

ROM_LIST tnzs_alt_romlist[] = {
   /* filename,      size,     checksum */
   { "NS_C-11.ROM",  128*1024, 0x6134e9ee },
   { "NS_E-3.ROM",   64*1024,  0x244e8a60 },

   { "NS_A04.ROM",   128*1024, 0x7c65f6df },
   { "NS_A07.ROM",   128*1024, 0x453ad1f8 },
   { "NS_A10.ROM",   128*1024, 0x274c4100 },
   { "NS_A13.ROM",   128*1024, 0x635a5412 },

   { "NS_A02.ROM",   128*1024, 0x2377128b },
   { "NS_A05.ROM",   128*1024, 0x186f4b07 },
   { "NS_A08.ROM",   128*1024, 0x6d0de83d },
   { "NS_A12.ROM",   128*1024, 0x68bed40a },
   { NULL }
};   

ROM_LIST tnzs_new_romlist[] = {
   /* filename,      size,     checksum */
   { "nz18",         64*1024,  0x0457390f },
   { "nz19",         64*1024,  0x8f030fbb },
   { "nz17",         64*1024,  0xcc5f8183 },

   { "nz2",          64*1024,  0xb3fab8ac },
   { "nz10",         64*1024,  0x5f2bad31 },
   { "nz4",          64*1024,  0x1b5e0a3c },
   { "nz12",         64*1024,  0x8832c62c },
   { "nz6",          64*1024,  0x4465123b },
   { "nz14",         64*1024,  0x5af8570a },
   { "nz8",          64*1024,  0x2d871843 },
   { "nz16",         64*1024,  0xfc5c0a72 },

   { "nz1",          64*1024,  0xa3ed27ad },
   { "nz9",          64*1024,  0xcf908dc8 },
   { "nz3",          64*1024,  0x59b5cbf1 },
   { "nz11",         64*1024,  0x6ecfd7f7 },
   { "nz5",          64*1024,  0x8b0eff5e },
   { "nz13",         64*1024,  0x794cb5e2 },
   { "nz7",          64*1024,  0xe6f8be50 },
   { "nz15",         64*1024,  0x24716c4b },
   { NULL }
};   

MACHINE tnzs_machine = {
   60, /* interrupts per second */
   256, 224, /* screen width, screen height */
   10, /* CPU interleaving */

   &TNZS_Init,
   &TNZS_Reset,
   &TNZS_Shutdown,
   &TNZS_DrawScreen,

   2, /* number of CPUs */
   { /* CPU settings */
      {
         8000000 /* 6 MHz */
      },
      {
         8000000 /* 6 MHz */
      }
   },

   1, /* number of types of audio CPUs used */
   { /* Audio chip settings */
      {
         &ym2203_interface,
         1,       /* 1 chip */
         1500000, /* 1.500000 MHz ??? */
      }
   }
};

GAME tnzs_driver = {
   "The New Zealand Story",
   "tnzs",
   &tnzs_machine,
   tnzs_romlist
};

GAME tnzs_alt_driver = {
   "The New Zealand Story (alternate)",
   "tnzs_alt",
   &tnzs_machine,
   tnzs_alt_romlist
};

GAME tnzs_new_driver = {
   "The New Zealand Story (new ROM set)",
   "tnzs_new",
   &tnzs_machine,
   tnzs_new_romlist
};
