#include <stdlib.h>
#include <string.h>
#include <clock.h>
#include "emu.h"
#include "utils.h"
#include "dip.h"

byte sound_latch;
int current_cpu, interleave;

int Emu_DoFrame(void)
{
   int n,t;
   CPU *cpu;

   poll_joystick();

   /* Execute one frame for each CPU */
   interleave = machine->interleave;
   do {
      Clock_Start(CLOCK_CPU);
      n = machine->num_cpus;
      cpu = &(machine->cpu[0]);
      for (current_cpu=0;current_cpu<n;current_cpu++,cpu++)
      {
         z80_set_context(cpu->regs);
         t = Z80_Interrupt();
         if (t != Z80_IGNORE_INT)
         {
            z80_raise_IRQ(t);
            z80_emulate(200);
            cpu->icount -= z80_get_cycles_elapsed();
            z80_lower_IRQ();
         }
         z80_emulate(cpu->icount);
         cpu->icount -= z80_get_cycles_elapsed();
         cpu->icount += cpu->iperiod;
         z80_get_context(cpu->regs);
      }

      Clock_Stop(CLOCK_CPU);
      Emu_UpdateAudio();
   } while(--interleave);

   percent_counter++;
   return TRUE;
}

int Emu_Reset(WINDOW *window, GADGET *d)
{
   int n, num_types;
   AUDIO_CHIP *chip;

   z80_reset();
   for (current_cpu=0;current_cpu<machine->num_cpus;current_cpu++)
   {
      z80_get_context(machine->cpu[current_cpu].regs);
      machine->cpu[current_cpu].icount = machine->cpu[current_cpu].iperiod;
   }

   machine->Reset();

   Emu_RequestRedraw();
   Palette_Recalc(&machine->cpu[0].image[0xf800]);

   /* reset audio system */
   num_types = machine->num_audio_chip_types;
   if (num_types)
   {
      chip = &(machine->audio_chip_types[0]);
      do {
         for (n=0;n<chip->num;n++)
         {
            chip->driver->Reset(n);
         }
         chip++;
      } while(--num_types);
   }

   GUI_PrintLog(FALSE, "Emulation reset.");
   return D_O_K;
}

int Z80_Interrupt(void)
{
   if (interleave == machine->interleave)
      return 0xff;
   else
      return Z80_IGNORE_INT;
}
