#include "gui.h"

char gui_hex_filter(char key)
{
   if (key >= 'a' && key <= 'f') return key;
   if (key >= 'A' && key <= 'F') return key;
   if (key >= '0' && key <= '9') return key;
   return NULL;
}

char gui_dec_filter(char key)
{
   if (key >= '0' && key <= '9') return key;
   return NULL;
}

int gui_radio_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   GADGET *gadget;
   BITMAP *bmp;
   int w;
   int (*callback)(WINDOW *window, GADGET *d);

   switch(msg->type)
   {
   case MSG_START:
      callback = d->dp2;
      if (callback) return callback(window, d);
      return D_O_K;
   case MSG_CLICK:
      d->flags |= D_SELECTED|D_DIRTY;
      gadget = window->gadgets;
      while(gadget->proc)
      {
         if (gadget->proc == &gui_radio_proc && gadget->d1 == d->d1
             && gadget != d)
         {
            gadget->flags &= ~D_SELECTED;
            gadget->flags |= D_DIRTY;
         }
         gadget++;
      }
      callback = d->dp2;
      if (callback) return callback(window, d);
      return D_O_K;
   case MSG_WANTFOCUS: return D_WANTFOCUS;
   case MSG_DRAW:
      clear_to_color(d->bmp, GUI_BG_NORMAL);
      if (d->flags & D_SELECTED)
         bmp = gui_data[GUI_RADIO2].dat;
      else
         bmp = gui_data[GUI_RADIO1].dat;

      blit(bmp, d->bmp, 0, 0, 0, 0, bmp->w, bmp->h);

      text_mode(GUI_BG_NORMAL);
      gui_textout(d->bmp, d->dp, 13, 0, GUI_BLACK, FALSE);

      if (window->input_focus == d && window == active_window)
      {
         w = text_length(font, d->dp);
         GUI_DrawDottedRect(d->bmp, 12, 0, 12+w, text_height(font)-1);
      }
      return D_O_K;
   }
   return D_O_K;
}

int gui_frame_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int y;
   switch(msg->type)
   {
   case MSG_DRAW:
      y = text_height(font) / 2 - 1;
      GUI_DrawRect3D(FALSE, d->bmp, d->w - 1, d->h - 1, 0, y);
      GUI_DrawRect3D(FALSE, d->bmp, 1, y+1, d->w - 2, d->h - 2);
      putpixel(d->bmp, d->w - 2, y+1, GUI_BG_SHADOW);
      putpixel(d->bmp, 1, d->h - 2, GUI_BG_SHADOW);

      text_mode(GUI_BG_NORMAL);
      gui_textout(d->bmp, d->dp, 4, 0, GUI_BLACK, FALSE);

      return D_O_K;
   }
   return D_O_K;
}

int gui_bmp_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   switch(msg->type)
   {
   case MSG_DRAW:
      blit(d->dp, d->bmp, 0, 0,
                          0, 0,
                          d->w, d->h);
      return D_O_K;
   }
   return D_O_K;
}

int gui_line_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   switch(msg->type)
   {
   case MSG_DRAW:
      GUI_DrawLine3D(FALSE, d->bmp, 0, 0, d->w-1, 0);
      return D_O_K;
   }
   return D_O_K;
}

int gui_text_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   switch(msg->type)
   {
   case MSG_DRAW:
      text_mode(GUI_BG_NORMAL);
      gui_textout(d->bmp, d->dp, 2, 2, GUI_BLACK, FALSE);
      return D_O_K;
   }
   return D_O_K;
}

int gui_button_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int tx, ty, n;
   GADGET *gadget;
   int (*func)(WINDOW *window, GADGET *d);

   switch(msg->type)
   {
   case MSG_START:
      d->d1 = FALSE;
      d->flags &= ~D_DRAGGING;
      return D_O_K;
   case MSG_WANTFOCUS:
      if (d->flags & D_DISABLED) return D_O_K;
      else return D_WANTFOCUS;
   case MSG_CLICK:
      if (d->flags & D_DISABLED) return D_O_K;

      d->flags |= D_DRAGGING|D_SELECTED|D_DIRTY;

      return D_O_K;
   case MSG_IDLE:
      if (d->flags & D_DRAGGING)
      {
         gadget = GUI_GetGadgetAtCoords(window,
                                         mouse_x - window->x,
                                         mouse_y - window->y);

         /* if mouse is hovering back over the button, redraw it selected */
         if (gadget == d && !(d->flags & D_SELECTED))
         {
            d->flags |= D_DIRTY|D_SELECTED;
         }

         /* if mouse is not hovering over the button, redraw it unselected */
         if (gadget != d && (d->flags & D_SELECTED))
         {
            d->flags |= D_DIRTY;
            d->flags &= ~D_SELECTED;
         }

         /* if they let go of the button, see if the let go over the button */
         if (!mouse_b)
         {
            d->flags &= ~(D_SELECTED|D_DRAGGING);
            d->flags |= D_DIRTY;

            if (d->dp2 && gadget == d)
            {
               func = d->dp2;
               return func(window, d);
            }

            if (gadget == d && (d->flags & D_EXIT))
            {
               return D_CLOSE;
            }
         }
      }
      return D_O_K;
   case MSG_DRAW:
      tx = d->w / 2;
      ty = (d->h - text_height(font)) / 2;

      if (d == window->input_focus && window == active_window)
      {
         rect(d->bmp, 0, 0, d->w - 1, d->h - 1, GUI_BLACK);
         n = 1;
      } else {
         n = 0;
      }

      rectfill(d->bmp, 1, 1, d->w - 2, d->h - 2, GUI_BG_NORMAL);

      if (d->flags & D_SELECTED)
         GUI_DrawRect3D(FALSE, d->bmp, d->w-n - 1, d->h-n - 1, n, n);
      else
         GUI_DrawRect3D(FALSE, d->bmp, n, n, d->w-n - 1, d->h-n - 1);

      text_mode(GUI_BG_NORMAL);
      if (d->flags & D_DISABLED)
         gui_textout(d->bmp, d->dp, tx, ty, GUI_DISABLED_TEXT, TRUE);
      else
         gui_textout(d->bmp, d->dp, tx, ty, GUI_BLACK, TRUE);

      return D_O_K;
   }

   return D_O_K;
}

int gui_closebox_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int cx, cy;
   switch(msg->type)
   {
   default: return gui_button_proc(msg, window, d);
   case MSG_WANTFOCUS: return D_O_K;
   case MSG_DRAW:
      rectfill(d->bmp, 1, 1, d->w - 2, d->h - 2, GUI_BG_NORMAL);

      if (d->flags & D_SELECTED)
         GUI_DrawRect3D(FALSE, d->bmp, d->w - 1, d->h - 1, 0, 0);
      else
         GUI_DrawRect3D(FALSE, d->bmp, 0, 0, d->w - 1, d->h - 1);

      cx = d->w / 2;
      cy = d->h / 2;

      if (d->flags & D_DISABLED)
      {
         line(d->bmp, cx-2, cy-2, cx+2, cy+2, GUI_DISABLED_TEXT);
         line(d->bmp, cx-2, cy+2, cx+2, cy-2, GUI_DISABLED_TEXT);
      } else {
         line(d->bmp, cx-2, cy-2, cx+2, cy+2, GUI_BLACK);
         line(d->bmp, cx-2, cy+2, cx+2, cy-2, GUI_BLACK);
      }

      return D_O_K;
   }

   return D_O_K;
}

int gui_scrollbar_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int new_d1, y;
   int (*func)(WINDOW *window, GADGET *d);

   switch(msg->type)
   {
   case MSG_START:
      d->flags &= ~D_DRAGGING;
      return D_O_K;
   case MSG_CLICK:
      if (d->flags & D_DISABLED) return D_O_K;

      d->flags |= D_DRAGGING|D_DIRTY;

      return D_O_K;
   case MSG_IDLE:
      if (d->flags & D_DRAGGING)
      {
         y = mouse_y - window->y - d->y;

         if (y < 0) y = 0;
         if (y > d->h - 16) y = d->h - 16;

         new_d1 = (y * d->d2) / (d->h - 16);
         if (new_d1 != d->d1)
         {
            d->flags |= D_DIRTY;
            d->d1 = new_d1;
            func = d->dp;
            func(window, d);
         }

         if (!mouse_b) d->flags &= ~D_DRAGGING;
      }
      return D_O_K;
   case MSG_DRAW:
      rectfill(d->bmp, 1, 1, d->w - 2, d->h - 2, GUI_BG_NORMAL);
      GUI_DrawRect3D(FALSE, d->bmp, 0, 0, d->w - 1, d->h - 1);

      y = (d->d1 * (d->h - 16)) / d->d2;
      GUI_DrawRect3D(FALSE, d->bmp, 0, y, d->w - 1, y+15);

      return D_O_K;
   }

   return D_O_K;
}

int gui_editbox_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int start, end, tx, ty, w, colour;
   char *str, buf[2], newkey;
   char (*filter)(char key);
   int (*callback)(WINDOW *window, GADGET *d);

   str = (char *)(d->dp);
   filter = d->dp2;

   switch(msg->type)
   {
   case MSG_START:
      d->d2 = strlen(str);
      callback = d->dp3;
      if (callback) return callback(window, d);
      return D_O_K;
   case MSG_WANTFOCUS:
      if (d->flags & D_DISABLED) return D_O_K;
      else return D_WANTFOCUS;
   case MSG_CLICK:
      if (d->flags & D_DISABLED) return D_O_K;
      return D_O_K;
   case MSG_IDLE:
      return D_O_K;
   case MSG_CHAR:
      d->flags |= D_DIRTY;
      callback = d->dp3;
      switch(msg->key >> 8)
      {
      case KEY_BACKSPACE:
         d->d2--;
         if (d->d2 < 0) d->d2 = 0;
         str[d->d2] = NULL;
         if (callback) return callback(window, d);
         else return D_O_K;
      }
      if (d->d2 == d->d1) return D_O_K;

      if (filter) newkey = filter(msg->key & 0xff);
      else newkey = msg->key & 0xff;

      if (newkey != NULL)
      {
         str[d->d2] = newkey;
         d->d2++;
      }

      if (callback) return callback(window, d);
      else return D_O_K;
   case MSG_DRAW:
      if (d->flags & D_DISABLED)
      {
         text_mode(GUI_BG_NORMAL);
         rectfill(d->bmp, 1, 1, d->w - 2, d->h - 2, GUI_BG_NORMAL);
      } else {
         text_mode(GUI_WHITE);
         rectfill(d->bmp, 1, 1, d->w - 2, d->h - 2, GUI_WHITE);
      }

      GUI_DrawRect3D(FALSE, d->bmp, d->w - 1, d->h - 1, 0, 0);

      tx = 2;
      ty = (d->h - text_height(font)) / 2;

      if (*str == NULL)
      {
         if (window->input_focus == d && window == active_window)
            hline(d->bmp, tx, ty + text_height(font) - gui_font_baseline,
                          tx + 4, GUI_BLACK);
         return D_O_K;
      }

      start = end = strlen(str) - 1;
      w = 0;

      if (d->d2 > end) w += 4; /* add the length of the cursor */

      buf[1] = NULL;
      while(1)
      {
         buf[0] = str[start];
         w += text_length(font, buf);
         if (w >= d->w - 4)
         {
            start++;
            break;
         }
         if (start == 0) break;
         start--;
      }

      if (d->flags & D_DISABLED)
         colour = GUI_DISABLED_TEXT;
      else
         colour = GUI_BLACK;

      str = &str[start];
      while (start <= end)
      {
         buf[0] = *str;
         textout(d->bmp, font, &buf[0], tx, ty, colour);

         w = text_length(font, buf);

         if (start == d->d2)
         {
            hline(d->bmp, tx, ty + text_height(font) - gui_font_baseline,
                          tx + w, colour);
         }

         tx += w;
         start++;
         str++;
      }

      if (d->d2 > end)
      {
         if (window->input_focus == d && window == active_window)
            hline(d->bmp, tx, ty + text_height(font) - gui_font_baseline,
                          tx + 4, GUI_BLACK);
      }
      return D_O_K;
   }

   return D_O_K;
}

int gui_window_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int newx, newy;
   switch(msg->type)
   {
   case MSG_START:
      d->flags &= ~D_DRAGGING;
      return D_O_K;
   case MSG_CLICK:
      if (msg->y >= 14) return D_O_K;
      d->flags |= D_DRAGGING;
      d->d1 = msg->x + window->x;
      d->d2 = msg->y + window->y;
      return D_O_K;
   case MSG_IDLE:
      if (d->flags & D_DRAGGING)
      {
         newx = mouse_x;
         newy = mouse_y;
         if (newx != d->d1 ||
             newy != d->d2)
         {
            GUI_MoveWindow(window, window->x + newx - d->d1,
                                    window->y + newy - d->d2);
            d->d1 = newx;
            d->d2 = newy;
         }
         if (!mouse_b) d->flags &= ~D_DRAGGING;
      }
      return D_O_K;
   case MSG_DRAW:
      if (window == active_window)
      {
         rectfill(d->bmp, 1, 1, d->w - 2, d->h - 2, GUI_FG_NORMAL);
         GUI_DrawRect3D(TRUE, d->bmp, 0, 0, d->w - 1, d->h - 1);
         GUI_DrawLine3D(TRUE, d->bmp, 1, 13, d->w - 2, 13);

         GUI_DrawRect3D(TRUE, d->bmp, d->w - 4, d->h - 4, 3, 17);
         rectfill(d->bmp, 4, 18, d->w - 5, d->h - 5, GUI_BG_NORMAL);

         text_mode(GUI_FG_NORMAL);
         textout(d->bmp, font, d->dp, 3, 1, GUI_FG_TEXT);
      } else {
         rectfill(d->bmp, 1, 1, d->w - 2, d->h - 2, GUI_BG_NORMAL);
         GUI_DrawRect3D(FALSE, d->bmp, 0, 0, d->w - 1, d->h - 1);
         GUI_DrawLine3D(FALSE, d->bmp, 1, 13, d->w - 2, 13);

         GUI_DrawRect3D(FALSE, d->bmp, d->w - 4, d->h - 4, 3, 17);
         rectfill(d->bmp, 4, 18, d->w - 5, d->h - 5, GUI_BG_NORMAL);

         text_mode(GUI_BG_NORMAL);
         textout(d->bmp, font, d->dp, 3, 1, GUI_BG_TEXT);
      }

      return D_O_K;
   }

   return D_O_K;
}
