#include <stdlib.h>
#include <string.h>
#include "emu.h"
#include "utils.h"

char memdump_address[5], memdump_cpu[5];
WINDOW *memdump_window=NULL;

GADGET gui_memdump_dialog[] = {
   /* (dialog proc)       (x)              (y)         (w)           (h)          (flags)  (d1) (d2)    (dp) [dp2] */
   { gui_window_proc,     0,               0,          NC_W+256+6*8, NC_H+128+22, 0,       0,   0,      "Memory dump" },
   { gui_closebox_proc,   256+6*8+NC_W-12, 2,          9,            9,           D_EXIT,  0,   0,      NULL },
   { gui_memdump_proc,    NC_X,            NC_Y,       256+6*8,      128,         0,       0,   0,      NULL },
   { gui_line_proc,       NC_X,            NC_Y+128,   256+6*8,      2,           0,       0,   0,      NULL },

   /* Address: */
   { gui_editbox_proc,    NC_X+50,         NC_Y+128+4, 50,           16,          0,       4,   0,      &memdump_address, &gui_hex_filter, &MemDump_Update },
   { gui_text_proc,       NC_X+2,          NC_Y+128+4, 45,           16,          0,       0,   0,      "Address:" },

   /* CPU: */
   { gui_editbox_proc,    NC_X+150,        NC_Y+128+4, 50,           16,          0,       4,   0,      &memdump_cpu, &gui_dec_filter, &MemDump_Update },
   { gui_text_proc,       NC_X+102,        NC_Y+128+4, 45,           16,          0,       0,   0,      "CPU:" },
   { NULL }
};

#define MEMDUMP_VIEWER  gui_memdump_dialog[2]
#define MEMDUMP_ADDRESS gui_memdump_dialog[4]
#define MEMDUMP_CPU     gui_memdump_dialog[6]

/* DP=image, D1=address */
int gui_memdump_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int start, x, y;
   switch(msg->type)
   {
   case MSG_START:
      d->d1 = 0x0000;
      if (d->dp)
      {
         MEMDUMP_ADDRESS.flags &= ~D_DISABLED;
         MEMDUMP_CPU.flags &= ~D_DISABLED;
      } else {
         MEMDUMP_ADDRESS.flags |= D_DISABLED;
         MEMDUMP_CPU.flags |= D_DISABLED;
      }
      return D_O_K;
   case MSG_DRAW:
      if (!d->dp)
      {
         clear_to_color(d->bmp, GUI_BG_NORMAL);
         return D_O_K;
      }

      text_mode(GUI_BG_NORMAL);
      start = d->d1;
      for (y=0;y<16;y++)
      {
         textprintf(d->bmp, bios_font, 0, y*8, GUI_BLACK,
                    "%.4X:", start);
         for (x=0;x<16;x++,start++)
         {
            if (x&1)
            {
               textprintf(d->bmp, bios_font, x*16+6*8, y*8, GUI_BG_HIGHLIGHT,
                          "%.2x", ((byte *)(d->dp))[start]);
            } else {
               textprintf(d->bmp, bios_font, x*16+6*8, y*8, GUI_BG_SHADOW,
                          "%.2x", ((byte *)(d->dp))[start]);
            }
         }
      }
      return D_O_K;
   case MSG_IDLE:
      if (d->dp)
      {
         if (MEMDUMP_ADDRESS.flags & D_DISABLED)
         {
            MEMDUMP_ADDRESS.flags &= ~D_DISABLED;
            MEMDUMP_ADDRESS.flags |= D_DIRTY;
         }

         if (MEMDUMP_CPU.flags & D_DISABLED)
         {
            MEMDUMP_CPU.flags &= ~D_DISABLED;
            MEMDUMP_CPU.flags |= D_DIRTY;
         }
         d->flags |= D_DIRTY;
      } else {
         if (!(MEMDUMP_ADDRESS.flags & D_DISABLED))
         {
            MEMDUMP_ADDRESS.flags |= D_DIRTY|D_DISABLED;
         }
         if (!(MEMDUMP_CPU.flags & D_DISABLED))
         {
            MEMDUMP_CPU.flags |= D_DIRTY|D_DISABLED;
         }
      }
      return D_O_K;
   }
   return D_O_K;
}

int MemDump_Init(WINDOW *window)
{
   int x,y;

   if (memdump_window)
   {
      GUI_SwitchFocus(memdump_window);
      return D_O_K;
   }

   /* create the memory dump window */
   GUI_PrintLog(FALSE, "Creating memory-dump window...");
   strcpy(memdump_address, get_config_string("Memory_dump", "address", "0000"));
   strcpy(memdump_cpu, get_config_string("Memory_dump", "cpu", "0"));
   x = get_config_int("Memory_dump", "x_position", 304);
   y = get_config_int("Memory_dump", "y_position", 18);
   memdump_window = GUI_NewWindow(x, y, gui_memdump_dialog, &MemDump_Shutdown);
   if (!memdump_window)
      return D_O_K;

   MemDump_Update(NULL, NULL);

   return D_O_K;
}

int MemDump_Update(WINDOW *window, GADGET *d)
{
   char *t;
   int addr, cpu;
   addr = strtol(memdump_address, &t, 16);
   if (addr < 0x0000) addr = 0x0000;
   if (addr > 0xff00) addr = 0xff00;

   if (current_game)
   {
      cpu = strtol(memdump_cpu, &t, 10);
      if (cpu < 0) cpu = 0;
      if (cpu >= machine->num_cpus) cpu = machine->num_cpus - 1;

      //if (MEMDUMP_VIEWER.d1 != addr)
      MEMDUMP_VIEWER.flags |= D_DIRTY;

      MEMDUMP_VIEWER.dp = machine->cpu[cpu].image;
   } else {
      MEMDUMP_VIEWER.dp = NULL;
   }
   MEMDUMP_VIEWER.d1 = addr;
   return D_O_K;
}

void MemDump_Shutdown(WINDOW *window)
{
   GUI_PrintLog(FALSE, "Shutting down memory-dump window.");
   set_config_int("Memory_dump", "x_position", window->x);
   set_config_int("Memory_dump", "y_position", window->y);
   set_config_string("Memory_dump", "address", memdump_address);
   set_config_string("Memory_dump", "cpu", memdump_cpu);
   memdump_window = NULL;
}
