#include <clock.h>
#include "emu.h"
#include "utils.h"

WINDOW *perfmon_window=NULL;
volatile int percent_count, percent_counter, fps_count, frame_count;
volatile int need_perfmon_update;
int reset_perfmon;

GADGET gui_perfmon_dialog[] = {
   /* (dialog proc)       (x)              (y)         (w)           (h)          (flags)  (d1) (d2)    (dp) [dp2] */
   { gui_window_proc,     0,               0,          NC_W+107,     NC_H+57,     0,       0,   0,      "Performance Monitor" },
   { gui_closebox_proc,   107+NC_W-12,     2,          9,            9,           D_EXIT,  0,   0,      NULL },
   { gui_perfmon_proc,    NC_X,            NC_Y,       107,          57,          0,       0,   0,      NULL },
   { NULL }
};

void perfmon_timer(void)
{
   percent_count = percent_counter;
   fps_count = frame_count;
   percent_counter = frame_count = 0;
   need_perfmon_update = TRUE;
}
END_OF_FUNCTION(perfmon_timer);

int gui_perfmon_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int n,total,val,c,y;
   char *name;
   switch(msg->type)
   {
   case MSG_START:
      LOCK_VARIABLE(fps_count);
      LOCK_VARIABLE(frame_count);
      LOCK_VARIABLE(percent_count);
      LOCK_VARIABLE(percent_counter);
      LOCK_FUNCTION(perfmon_timer);
      install_int_ex(&perfmon_timer, BPS_TO_TIMER(1));

      Clock_Init();

      frame_count = fps_count = percent_count = percent_counter = 0;
      need_perfmon_update = reset_perfmon = TRUE;
      return D_O_K;
   case MSG_END:
      remove_int(&perfmon_timer);
      Clock_Exit();

      return D_O_K;
   case MSG_IDLE:
      if (need_perfmon_update)
      {
         d->flags |= D_DIRTY;
         need_perfmon_update = FALSE;
      }
      if (reset_perfmon)
      {
         Clock_Reset();
         reset_perfmon = FALSE;
      }

      return D_O_K;
   case MSG_DRAW:
      clear_to_color(d->bmp, GUI_BG_NORMAL);
      text_mode(-1);

      if (current_game)
      {
         c = GUI_BLACK;
         textprintf(d->bmp, font, 2, 2, c, "FPS=%i", fps_count);
         textprintf(d->bmp, font, 52, 2, c, "Speed=%i%%",
                    100 * percent_count / machine->interrupt_rate);
      } else {
         c = GUI_DISABLED_TEXT;
         textprintf(d->bmp, font, 2, 2, c, "FPS=N/A");
         textprintf(d->bmp, font, 52, 2, c, "Speed=N/A");
      }

      total = 0;
      for (n=0;n<NUM_CLOCKS;n++)
      {
         total += Clock[n].total;
      }

      y = 18;
      for (n=0;n<NUM_CLOCKS;n++)
      {
         switch(n)
         {
         case CLOCK_CPU: name = "CPU:"; break;
         case CLOCK_RENDER: name = "Render:"; break;
         case CLOCK_AUDIO: name = "Audio:"; break;
         default: name = NULL;
         }
         textprintf(d->bmp, font, 2, y, c, name);
         GUI_DrawRect3D(FALSE, d->bmp, 102, y+10, 52, y);
         y += 12;
      }

      if (total != 0 && current_game)
      {
         y = 18;
         for (n=0;n<NUM_CLOCKS;n++)
         {
            val = 50 * Clock[n].total / total;
            rectfill(d->bmp, 53, y+1, 53+val, y+9, GUI_URGENT_TEXT);
            y += 12;
         }
      }
      reset_perfmon = TRUE;
      return D_O_K;
   }
   return D_O_K;
}

int PerfMon_Init(WINDOW *window)
{
   int x,y;

   if (perfmon_window)
   {
      GUI_SwitchFocus(perfmon_window);
      return D_O_K;
   }

   /* create the monitor window */
   GUI_PrintLog(FALSE, "Creating performance-monitor window...");
   x = get_config_int("Performance_monitor", "x_position", 361);
   y = get_config_int("Performance_monitor", "y_position", 391);
   perfmon_window = GUI_NewWindow(x, y, gui_perfmon_dialog, &PerfMon_Shutdown);

   return D_O_K;
}

void PerfMon_Shutdown(WINDOW *window)
{
   GUI_PrintLog(FALSE, "Shutting down performance-monitor window.");
   set_config_int("Performance_monitor", "x_position", window->x);
   set_config_int("Performance_monitor", "y_position", window->y);
   perfmon_window = NULL;
}
