#include <string.h>
#include "nz.h"
#include "dip.h"

char *startup_game;
FONT *bios_font;
int use_scanlines;

GADGET gui_toolbar_dialog[] = {
   /* (dialog proc)    (x)     (y)       (w)       (h)           (flags)  (d1) (d2)  (dp) [dp2] */
   { gui_window_proc,  0,      0,        NC_W+120, NC_H+400,     0,       0,   0,    "TNZS-EMU " VERSION_STRING "" },
   { gui_button_proc,  NC_X+6, NC_Y+6,   100,      30,           0,       0,   0,    "Change game", &Main_ChangeGame },
   { gui_button_proc,  NC_X+6, NC_Y+46,  100,      30,           0,       0,   0,    "Full-screen", &Main_Fullscreen },
   { gui_button_proc,  NC_X+6, NC_Y+86,  100,      30,           0,       0,   0,    "Reset emulation", &Emu_Reset },
   { gui_button_proc,  NC_X+6, NC_Y+126, 100,      30,           0,       0,   0,    "DIP switches", &Emu_DipSwitches },
   { gui_button_proc,  NC_X+6, NC_Y+166, 100,      30,           0,       0,   0,    "Options", &Emu_Options },
   { gui_button_proc,  NC_X+6, NC_Y+206, 100,      30,           0,       0,   0,    "Debugging options", &Emu_Debugging },
   { gui_button_proc,  NC_X+6, NC_Y+246, 100,      30,           0,       0,   0,    "Help/About", &Main_Help },
   { gui_button_proc,  NC_X+6, NC_Y+286, 100,      30,           0,       0,   0,    "Quit", &Main_Quit },
   { NULL }
};

BEGIN_COLOR_DEPTH_LIST
   COLOR_DEPTH_8
END_COLOR_DEPTH_LIST

BEGIN_DIGI_DRIVER_LIST
   DIGI_DRIVER_SOUNDSCAPE
   DIGI_DRIVER_AUDIODRIVE
   DIGI_DRIVER_SB
END_DIGI_DRIVER_LIST

BEGIN_MIDI_DRIVER_LIST
END_MIDI_DRIVER_LIST

int Main_ChangeGame(WINDOW *window, GADGET *d)
{
   int x, y;

   if (gamelist_window)
   {
      GUI_SwitchFocus(gamelist_window);
      return D_O_K;
   }

   /* create the game-list window */
   GUI_PrintLog(FALSE, "Creating game-list window.");
   x = get_config_int("Gamelist_window", "x_position", 230);
   y = get_config_int("Gamelist_window", "y_position", 170);
   gamelist_window = GUI_NewWindow(x, y, gui_gamelist_dialog, &Gamelist_Shutdown);

   return D_O_K;
}

void Main_StartGame(GAME *game)
{
   if (current_game != NULL)
   {
      /* disable buttons */
      gui_toolbar_dialog[2].flags |= D_DISABLED|D_DIRTY;
      gui_toolbar_dialog[3].flags |= D_DISABLED|D_DIRTY;
      gui_toolbar_dialog[4].flags |= D_DISABLED|D_DIRTY;
      Emu_Shutdown();
   }

   if (!Emu_Init(game))
   {
      Emu_Shutdown();
      return;
   }

   /* enable Fullscreen, Reset and DIP switch buttons */
   gui_toolbar_dialog[2].flags &= ~D_DISABLED;
   gui_toolbar_dialog[2].flags |= D_DIRTY;
   gui_toolbar_dialog[3].flags &= ~D_DISABLED;
   gui_toolbar_dialog[3].flags |= D_DIRTY;
   gui_toolbar_dialog[4].flags &= ~D_DISABLED;
   gui_toolbar_dialog[4].flags |= D_DIRTY;
}

int Main_Fullscreen(WINDOW *window, GADGET *d)
{
   BITMAP *bmp;
   int dx, dy, old_w, old_h, frames;

   /* set our new graphics mode */
   old_w = SCREEN_W;
   old_h = SCREEN_H;
   bmp = emu_window->gadgets[1].bmp;

   show_mouse(NULL);

   if (use_scanlines)
   {
      set_gfx_mode(GFX_MODEX, 320, 480, 0,0);
      screen->w = 320;
      screen->h = 240;
      screen->cr = screen->w;
      screen->cb = screen->h;
      gfx_driver->w = screen->w;
      gfx_driver->h = screen->h;
      for (dy=0;dy<240;dy++){
         screen->line[dy]=screen->line[dy*2];
      }
   } else {
      set_gfx_mode(GFX_AUTODETECT, 320, 240, 0,0);
   }
   set_palette(gui_palette);
   dx = (SCREEN_W - machine->gfx_w) / 2;
   dy = (SCREEN_H - machine->gfx_h) / 2;
   GUI_PrintLog(FALSE, "Fullscreen mode started.");

   while(!key[KEY_ESC])
   {
      Emu_CheckKeys(bmp);

      /* advance by however many frames */
      if (frameskip)
         frames = frameskip;
      else
         frames = timertic - gametic;

      if (!frames)
      {
         if (use_vsync) frames = 1;
         else continue;
      }
      if (frames > MAX_FRAME_SKIP) frames = MAX_FRAME_SKIP;

      do {
         Emu_UpdateAudio();
         if (!Emu_DoFrame()) return D_CLOSE;
      } while(--frames);
      gametic = timertic;

      /* redraw */
      Emu_UpdateAudio();
      machine->DrawScreen(bmp);
      if (emu_palette_dirty)
      {
         emu_palette_dirty = FALSE;
         set_palette(gui_palette);
      } else {
         if (use_vsync) vsync();
      }
      Emu_UpdateAudio();
      blit(bmp, screen, 0,0, dx,dy, machine->gfx_w, machine->gfx_h);
   }

   /* wait for the key-up */
   while(key[KEY_ESC]);
   clear_keybuf();
   
   /* set the old graphics mode and then redraw the screen*/
   set_gfx_mode(GFX_AUTODETECT, old_w, old_h, 0,0);
   set_palette(gui_palette);
   GUI_Refresh(NULL);

   show_mouse(screen);
   GUI_PrintLog(FALSE, "Fullscreen mode shutdown.");
   return D_O_K;
}

void Main_CloseToolbar(WINDOW *window)
{
   set_config_int("Toolbar_window", "x_position", window->x);
   set_config_int("Toolbar_window", "y_position", window->y);
}

int Main_Quit(WINDOW *window, GADGET *d)
{
   MSG msg;
   msg.type = MSG_END;
   GUI_PostMessage(&global_msg_queue, &msg);
   return D_O_K;
}

int Main_Shutdown(WINDOW *window, GADGET *d)
{
   /* save our options */
   if (use_vsync) set_config_string("Options", "use_vsync", "yes");
   else set_config_string("Options", "use_vsync", "no");
   set_config_int("Options", "frameskip", frameskip);

   if (use_scanlines) set_config_string("Options", "use_scanlines", "yes");
   else set_config_string("Options", "use_scanlines", "no");

   /* write out the key mappings */
   GUI_PrintLog(FALSE, "Saving key mappings.");
   set_config_int("Key_mappings", "left", key_left);
   set_config_int("Key_mappings", "right", key_right);
   set_config_int("Key_mappings", "up", key_up);
   set_config_int("Key_mappings", "down", key_down);
   set_config_int("Key_mappings", "button1", key_b1);
   set_config_int("Key_mappings", "button2", key_b2);
   set_config_int("Key_mappings", "start1", key_start1);
   set_config_int("Key_mappings", "start2", key_start2);
   set_config_int("Key_mappings", "coin", key_coin);
   return D_O_K;
}

int Main_Startup(void)
{
   int x, y;
   const char *str;

   current_game = NULL;
   /* Disable Reset and DIP switch buttons */
   gui_toolbar_dialog[2].flags |= D_DISABLED;
   gui_toolbar_dialog[3].flags |= D_DISABLED;
   gui_toolbar_dialog[4].flags |= D_DISABLED;

   /* read in the vsync setting */
   str = get_config_string("Options", "use_vsync", "no");
   use_vsync = FALSE;
   if (stricmp(str, "yes") == 0) use_vsync = TRUE;
   if (stricmp(str, "true") == 0) use_vsync = TRUE;

   /* read in the scanline setting */
   str = get_config_string("Options", "use_scanlines", "no");
   use_scanlines = FALSE;
   if (stricmp(str, "yes") == 0) use_scanlines = TRUE;
   if (stricmp(str, "true") == 0) use_scanlines = TRUE;

   /* read in the frameskip setting */
   frameskip = get_config_int("Options", "frameskip", 0);
   if (frameskip < 0) frameskip = 0;
   if (frameskip > MAX_FRAME_SKIP) frameskip = MAX_FRAME_SKIP;

   /* read in the key mappings */
   GUI_PrintLog(FALSE, "Reading key mappings.");
   key_left = get_config_int("Key_mappings", "left", KEY_LEFT);
   key_right = get_config_int("Key_mappings", "right", KEY_RIGHT);
   key_up = get_config_int("Key_mappings", "up", KEY_UP);
   key_down = get_config_int("Key_mappings", "down", KEY_DOWN);
   key_b1 = get_config_int("Key_mappings", "button1", KEY_LCONTROL);
   key_b2 = get_config_int("Key_mappings", "button2", KEY_ALT);
   key_start1 = get_config_int("Key_mappings", "start1", KEY_1);
   key_start2 = get_config_int("Key_mappings", "start2", KEY_2);
   key_coin = get_config_int("Key_mappings", "coin", KEY_3);

   /* set our main window */
   GUI_PrintLog(FALSE, "Initalising toolbar.");
   x = get_config_int("Toolbar_window", "x_position", 30);
   y = get_config_int("Toolbar_window", "y_position", 30);
   if (!GUI_NewWindow(x, y, gui_toolbar_dialog, &Main_CloseToolbar))
      return FALSE;

   /* create the emulation window */
   GUI_PrintLog(FALSE, "Creating emulation window.");
   x = get_config_int("Emulation_window", "x_position", 200);
   y = get_config_int("Emulation_window", "y_position", 30);
   emu_window = GUI_NewWindow(x, y, gui_emu_dialog, &EmuWindow_Shutdown);
   if (!emu_window) return FALSE;

   if (startup_game)
   {
      Gamelist_RunGame(startup_game);
   }

   return TRUE;
}

int main(int argc, char *argv[])
{
   const char *log;

   if (allegro_init() != 0)
   {
      fprintf(stderr, "Error initialising Allegro at startup!.\n");
      return 1;
   }
   set_config_file("tnzs.cfg");
   bios_font = font;

   /* First, see if we need to write a message log */
   log = get_config_string("Debugging", "write_log", "yes");
   if (stricmp(log, "yes") == 0 || stricmp(log, "true") == 0)
   {
      log_file = fopen(LOG_FILENAME, "w");
      GUI_PrintLog(FALSE, "Writing log to '" LOG_FILENAME "'.");
   } else {
      log_file = NULL;
   }

   GUI_PrintLog(FALSE, "TNZS-EMU " VERSION_STRING "");

   /* see if the user specified a game on the command line */
   if (argc > 1)
   {
      startup_game = argv[1];
      GUI_PrintLog(FALSE, "Game '%s' given via command line.", startup_game);
   } else {
      startup_game = NULL;
   }

   /* drag the GUI into life */
   if (!GUI_Init("TNZS.DAT")) return 1;
   GUI_PrintLog(FALSE, "Startup completed successfully.");

   GUI_SetStartupFunction(&Main_Startup, &Main_Shutdown);

   /* let it run */
   if (!GUI_Run()) return 1;

   if (log_file) fclose(log_file);
   printf("TNZS-EMU " VERSION_STRING ", Copyright (c) 1998 Richard Mitton\n"
          "Thank you for playing.\n"
          "See you again.\n");
   return 0;
}
